/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/base.cpp
    begin             : Wed Mar 26 2003
    $Id: base.cpp,v 1.5 2005/02/14 10:44:53 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

using namespace ufo;

// This is the base code used in most examples
// It initializes libUFO, creates a frame and creates a static text objects.

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// Creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// Creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("base code");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// this creates some widgets and adds them to the content pane
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
}
