/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/buttons.cpp
    begin             : Wed Mar 26 2003
    $Id: buttons.cpp,v 1.6 2005/06/21 09:28:47 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows the use of
// * push-buttons
// * check boxes
// * button groups with radio buttons

// * signals & slots


using namespace ufo;

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("button widgets");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// All changes compared to base.cpp are within the following method
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

// this method should be called when a button was clicked
void
button_clicked(UActionEvent * e) {
	std::cout << "button " << e->getSource() << " clicked\n";
}

void
button_toggled(UActionEvent * e) {
	UButton * b = dynamic_cast<UButton*>(e->getSource());
	if (b) {
		std::cout << "button " << e->getSource() << " toggled (now ";
		if (b->isSelected()) {
			std::cout << "selected)\n";
		} else  {
			std::cout << "deselected)\n";
		}
	}
}

void
createContent(UWidget * content) {
	content->setLayout(new UFlowLayout());

	//
	// push buttons
	//
	// This creates a simple push button with caption press me!
	// The character followin '&' is used as mnemonic key, i.e.
	// pressing Alt+p activates this button
	UButton * button = new UButton("&press me!");
	content->add(button);

	// If want want the button to to something on clicking, you have to
	// create a slot and connect it with the desired signal

	// connect the click signal with the method above
	button->sigActivated().connect(slot(&button_clicked));

	// add accelerator support
	// don't use blank spaces between modifiers and characters
	//button->setAccelerator("Alt+p");


	// Buttons with the same mnemonic key
	// If two buttons have the same mnemonic key, the button gets focused
	// but not activated. Pressing Alt+m again focuses the next button
	// with mnemonic key m.
	button = new UButton("same &mnemonic 1!");
	content->add(button);
	button = new UButton("same &mnemonic 2!");
	content->add(button);

	// disabled button
	button = new UButton("&disabled button");
	button->setEnabled(false);
	content->add(button);

	//
	// check boxes
	//
	UCheckBox * checkbox = new UCheckBox("&check box 1");
	content->add(checkbox);

	// A selected check box return true on
	// checkbox->isSelected()
	checkbox->sigActivated().connect(slot(&button_toggled));

	//
	// button groups
	//

	// A button group is a logical group which ensures that
	// only one button within the group is pressed
	UButtonGroup * group = new UButtonGroup();
	URadioButton * radioButton;
	// a group box is a visible grouping of elements with a title
	UGroupBox * box = new UGroupBox("radio group");
	box->setLayout(new UBoxLayout(Horizontal));
	//box->setEnabled(false);

	radioButton = new URadioButton("radio button: 1");

	// add the button to the button group
	group->addButton(radioButton);
	// and to the screen
	box->add(radioButton);

	// add two more
	radioButton = new URadioButton("radio &button: 2");
	group->addButton(radioButton);
	box->add(radioButton);
	// This signal is emitted every time the selection state of
	// this button (#2) is changed
	radioButton->sigActivated().connect(slot(&button_toggled));

	radioButton = new URadioButton("radio button: 3");
	group->addButton(radioButton);
	box->add(radioButton);

	content->add(box);
}
