/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/combo.cpp  -  description
    begin             : Wed Mar 26 2003
    $Id: combo.cpp,v 1.5 2005/02/14 10:48:26 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows the combo box, an editable box
// with several predefined choices to select.

using namespace ufo;

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("combo boxes");
	frame->setBounds(0, 0, 640, 480);
	frame->setResizable(true);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// this creates some widgets and adds them to the content pane
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add all widgets to the content pane
	content->add(new ULabel("fixed size combo box:"));

	// fixed size combo box
	UComboBox * combo = new UComboBox();
	content->add(combo);
	combo->getTextEdit()->setColumns(10);
	combo->addItem("hello, world");
	combo->addItem("item 2");
	combo->addItem("item 3");
	combo->addItem("item 4");
	combo->addItem("item 5");
	combo->addItem("item 6");
	combo->addItem("item 7");
	combo->addItem("item 8");
	combo->setCurrentItem(3);
	
	content->add(new ULabel("variable size combo box:"));
	// combo box with variable size (depending on the current item)
	UComboBox * var_combo = new UComboBox();
	content->add(var_combo);
	var_combo->addItem("var size combo box");
	var_combo->addItem("var combo item 2");
	var_combo->addItem("var combo item 3");
	var_combo->addItem("var combo item 4");
	var_combo->addItem("var combo item 5");
	var_combo->addItem("var combo item 6");
	var_combo->addItem("var combo item 7");
	var_combo->addItem("var combo item 8");
	var_combo->setCurrentItem(0);
	
	content->add(new ULabel("input disabled combo box:"));
	// combo box with variable size (depending on the current item)
	UComboBox * noentry_combo = new UComboBox();
	content->add(noentry_combo);
	noentry_combo->getTextEdit()->setEditable(false);
	noentry_combo->getTextEdit()->setColumns(10);
	noentry_combo->addItem("combo box");
	noentry_combo->addItem("item 2");
	noentry_combo->addItem("vitem 3");
	noentry_combo->addItem("item 4");
	noentry_combo->addItem("item 5");
	noentry_combo->addItem("item 6");
	noentry_combo->addItem("item 7");
	noentry_combo->addItem("item 8");
	noentry_combo->setCurrentItem(0);
}
