/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/base.cpp
    begin             : Wed Mar 26 2003
    $Id: css.cpp,v 1.1 2005/05/21 15:56:01 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

using namespace ufo;

// This example shows how to use CSS files to style libUFO

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// Creates the toolkit object, initializes UFO
	UXToolkit tk;

	// loads the video driver and creates the event queue
	UXDisplay display;

	// NEW: This loads the style sheet
	// be sure to call it after creating the toolkit and the display
	// (it may need system ressources in form of images)
	tk.getStyleManager()->loadStyleSheet("gui.css");

	// Creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("Cascading Style Sheets example");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// this creates some widgets and adds them to the content pane
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));

	//
	// Comments to CSS:
	// Cascading style sheets may built their style from
	//  several style entries.
	// In the given example (and with the distributed gui.css),
	// all labels first use the given label style. Then they look
	// for CSS classes (like colored, several etc. See gui.css).
	// All those entries determine the style of the widget.
	//

	UButton * button = new UButton("class id: colored");
	// this sets the CSS class id
	// in CSS, that means
	//  * look for button style
	//  * then look for a ".colored" entry
	button->setCssClass("colored");
	content->add(button);

	ULabel * label = new ULabel("class id: colored");
	label->setCssClass("colored");
	content->add(label);

	label = new ULabel("class id: several");
	label->setCssClass("several");
	content->add(label);
}
