/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/buttons.cpp
    begin             : Wed Mar 26 2003
    $Id: events.cpp,v 1.2 2005/02/14 10:45:03 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows the use of
// * mouse events
// * keyboard events

using namespace ufo;

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("mouse and keyboard events");
	frame->setBounds(0, 0, 640, 480);
	frame->setResizable(true);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// this creates some widgets and adds them to the content pane
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
print_all_mouse_events(UMouseEvent * e) {
	// if we got a button event, check for the button
	std::string button;
	switch (e->getButton()) {
		case UMod::LeftButton:
			button = "left mouse button";
			break;
		case UMod::RightButton:
			button = "right mouse button";
			break;
		case UMod::MiddleButton:
			button = "middle mouse button";
			break;
	}
	switch (e->getType()) {
		case UEvent::MouseEntered:
			std::cout << "mouse entered at " << e->getLocation() << "\n";
			break;
		case UEvent::MouseExited:
			std::cout << "mouse exited at " << e->getLocation() << "\n";
			break;
		case UEvent::MouseMoved:
			std::cout << "mouse moved at " << e->getLocation() << "\n";
			break;
		case UEvent::MouseDragged:
			std::cout << "mouse dragged at " << e->getLocation() << "\n";
			break;
		case UEvent::MousePressed:
			std::cout << button << " pressed at " << e->getLocation() << "\n";
			break;
		case UEvent::MouseReleased:
			std::cout << button << " released at " << e->getLocation() << "\n";
			break;
		case UEvent::MouseClicked:
			std::cout << button << " clicked at " << e->getLocation() << "\n";
			break;
		default: // this will never be executed (hopefully)
		break;
	}
}

void
print_all_key_events(UKeyEvent * e) {
	switch (e->getType()) {
		case UEvent::KeyPressed:
			std::cout << "key " << e->getKeyCode() << " pressed.\n";
			break;
		case UEvent::KeyReleased:
			std::cout << "key " << e->getKeyCode() << " released.\n";
			break;
		case UEvent::KeyTyped:
			std::cout << "unicode char \"" << char(e->getKeyChar()) << "\" typed.\n";
			break;
		default:
		break;
	}
}

void
focus_widget(UMouseEvent * e) {
	e->getWidget()->requestFocus();
}

void
createContent(UWidget * content) {
	content->setLayout(new UFlowLayout());

	//
	// mouse events
	//
	content->add(new ULabel("prints all mouse events:"));
	
	UWidget * mouse_events = new UWidget();
	// The default size of base widgets is 0.
	// This sets the preferred size used for the layout manager
	mouse_events->setPreferredSize(UDimension(100, 100));

	mouse_events->sigMouseEntered().connect(slot(print_all_mouse_events));
	mouse_events->sigMouseExited().connect(slot(print_all_mouse_events));
	mouse_events->sigMouseMoved().connect(slot(print_all_mouse_events));
	mouse_events->sigMouseDragged().connect(slot(print_all_mouse_events));
	
	mouse_events->sigMousePressed().connect(slot(print_all_mouse_events));
	mouse_events->sigMouseReleased().connect(slot(print_all_mouse_events));
	mouse_events->sigMouseClicked().connect(slot(print_all_mouse_events));
	
	// make it visible
	mouse_events->setBorder(LineBorder);
	
	content->add(mouse_events);
	
	
	//
	// mouse events
	//
	content->add(new ULabel("prints all keyboard events (click to focus):"));
	
	UWidget * key_events = new UWidget();
	// The default size of base widgets is 0.
	// This sets the preferred size used for the layout manager
	key_events->setPreferredSize(UDimension(100, 100));
	// Only the focused widget receives keyboard events
	key_events->sigMousePressed().connect(slot(focus_widget));
	
	key_events->sigKeyPressed().connect(slot(print_all_key_events));
	key_events->sigKeyReleased().connect(slot(print_all_key_events));
	key_events->sigKeyTyped().connect(slot(print_all_key_events));
	
	// make it visible
	key_events->setBorder(LineBorder);
	
	content->add(key_events);
}
