/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/gltest.cpp
    begin             : Wed Aug 4 2004
    $Id: gltest.cpp,v 1.6 2005/05/21 15:23:37 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

using namespace ufo;

// This example shows one way how to integrate custom OpenGL code
// in libUFO (another example is widget3d.cpp)

void createContent(UWidget * content);

void draw();

// Alternative way of redrawing. More complex but with more performance tweaks
void altRedraw(UXFrame * frame);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("custom opengl code");
	frame->setBounds(0, 0, 640, 480);
	frame->setResizable(true);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// this creates some widgets and adds them to the content pane
	createContent(frame->getContentPane());

	// NEW
	// this gives a performance boost and prevents libUFO to draw
	// above your GL code
	frame->getRootPane()->setOpaque(false);

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// if you uncomment altRedraw, comment out draw and frame.repaint
			//altRedraw(frame);
			draw();
			// repaint the frame
			// the argument forces a repaint
			frame->repaint(true);
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new UButton("useless button"));
}


// we need the opengl headers
#include <GL/gl.h>

// copied from testgl.c of test directory of the SDL1.2.3 sources
// http://www.libsdl.org
static const float color[8][3]= {
	{ 1.0,  1.0,  0.0},
	{ 1.0,  0.0,  0.0},
	{ 0.0,  0.0,  0.0},
	{ 0.0,  1.0,  0.0},
	{ 0.0,  1.0,  1.0},
	{ 1.0,  1.0,  1.0},
	{ 1.0,  0.0,  1.0},
	{ 0.0,  0.0,  1.0}
};
static const float cube[8][3]= {
	{ 0.5,  0.5, -0.5},
	{ 0.5, -0.5, -0.5},
	{-0.5, -0.5, -0.5},
	{-0.5,  0.5, -0.5},
	{-0.5,  0.5,  0.5},
	{ 0.5,  0.5,  0.5},
	{ 0.5, -0.5,  0.5},
	{-0.5, -0.5,  0.5}
};


void
initProjection() {
	// set up projection and model view matrix
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glOrtho(-2.0, 2.0, -2.0, 2.0, -20.0, 20.0);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// rotate (for rotating cube)
	static float rot = 0.0f;
	glRotatef(rot, 1.0, 1.0, 1.0);
	rot += 5.0f;

	if (rot >360.0f) {
		rot = 0.0f;
	}
}

void
resetProjection() {
	// reset projection and model view matrix
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void
draw3D() {
	// draw a colored cube
	glBegin(GL_QUADS);

	glColor3fv(color[0]);
	glVertex3fv(cube[0]);
	glColor3fv(color[1]);
	glVertex3fv(cube[1]);
	glColor3fv(color[2]);
	glVertex3fv(cube[2]);
	glColor3fv(color[3]);
	glVertex3fv(cube[3]);

	glColor3fv(color[3]);
	glVertex3fv(cube[3]);
	glColor3fv(color[4]);
	glVertex3fv(cube[4]);
	glColor3fv(color[7]);
	glVertex3fv(cube[7]);
	glColor3fv(color[2]);
	glVertex3fv(cube[2]);

	glColor3fv(color[0]);
	glVertex3fv(cube[0]);
	glColor3fv(color[5]);
	glVertex3fv(cube[5]);
	glColor3fv(color[6]);
	glVertex3fv(cube[6]);
	glColor3fv(color[1]);
	glVertex3fv(cube[1]);

	glColor3fv(color[5]);
	glVertex3fv(cube[5]);
	glColor3fv(color[4]);
	glVertex3fv(cube[4]);
	glColor3fv(color[7]);
	glVertex3fv(cube[7]);
	glColor3fv(color[6]);
	glVertex3fv(cube[6]);

	glColor3fv(color[5]);
	glVertex3fv(cube[5]);
	glColor3fv(color[0]);
	glVertex3fv(cube[0]);
	glColor3fv(color[3]);
	glVertex3fv(cube[3]);
	glColor3fv(color[4]);
	glVertex3fv(cube[4]);

	glColor3fv(color[6]);
	glVertex3fv(cube[6]);
	glColor3fv(color[1]);
	glVertex3fv(cube[1]);
	glColor3fv(color[2]);
	glVertex3fv(cube[2]);
	glColor3fv(color[7]);
	glVertex3fv(cube[7]);

	glEnd();
}


void
draw() {
	glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);

	glPushAttrib(GL_LIGHTING_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);

	initProjection();
	draw3D();
	resetProjection();

	// reset attributes
	glPopAttrib();
}
