/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/glut.cpp
    begin             : Wed Aug 4 2004
    $Id: glut.cpp,v 1.6 2005/02/14 10:45:03 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows how to use Glut directly

// this is for convenience. The UXSDLDevice provides SDL event conversion.
//#include <ufo/ux/uxsdldevice.hpp>

#ifdef HAVE_GLUT_H

#include <GL/glut.h>

using namespace ufo;

void createContent(UWidget * content);

UXDisplay * display;
UXContext * context;

// event callbacks
void mouseMotionFunc(int x, int y);
void mouseButtonFunc(int button, int updown, int x, int y) ;

void keyboardFunc(unsigned char key, int, int);
void keyboardUpFunc(unsigned char key, int, int);

void specialFunc(int key, int, int);
void specialUpFunc(int key, int, int);

// display callbacks
void
displayFunc() {
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f) ;
	glClear(GL_COLOR_BUFFER_BIT);

	context->repaint();

	glutSwapBuffers();
}

void
idleFunc() {
	display->dispatchEvents();

	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}

void
reshapeFunc(int width, int height) {
	context->setDeviceBounds(URectangle(0, 0, width, height));
	glutPostRedisplay();
}

int
main(int argc, char ** argv) {
	// init SDL
	glutInitWindowSize(640, 480) ;
	glutInit(&argc, argv) ;

	/* UFO does not need a depth buffer.
	 * But most user apps do some complex 3D rendering requiring a depth buffer.
	 */
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH) ;

	glutCreateWindow("UFO Application");

	// set-up callbacks
	glutDisplayFunc(displayFunc);
	glutReshapeFunc(reshapeFunc);

	glutMouseFunc(mouseButtonFunc);
	glutMotionFunc(mouseMotionFunc);
	glutPassiveMotionFunc(mouseMotionFunc);

	glutKeyboardFunc(keyboardFunc);
	glutKeyboardUpFunc(keyboardUpFunc);
	glutSpecialFunc(specialFunc);
	glutSpecialUpFunc(specialUpFunc);


	// init UFO
	UXToolkit tk;
	// init the display with a dummy driver
	display = new UXDisplay("dummy");

	// create a context object (without frame).
	context = new UXContext(
		// bounds of the OpenGL device
		URectangle(0, 0, 640, 480),
		// desired bounds of the UFO context (may differ from above values)
		URectangle(20, 20, 400, 400)
	);

	createContent(context->getRootPane()->getContentPane());

	glutIdleFunc(idleFunc);
	glutMainLoop();

	delete(context);
	delete(display);

	return EXIT_SUCCESS;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
	content->add(new UButton("button"));
}

// map special keys
UKeyCode_t
mapGlutSpecialKey(int key) {
	switch (key) {
		// function keys
		case GLUT_KEY_F1: return UKey::UK_F1;
		case GLUT_KEY_F2: return UKey::UK_F2;
		case GLUT_KEY_F3: return UKey::UK_F3;
		case GLUT_KEY_F4: return UKey::UK_F4;
		case GLUT_KEY_F5: return UKey::UK_F5;
		case GLUT_KEY_F6: return UKey::UK_F6;
		case GLUT_KEY_F7: return UKey::UK_F7;
		case GLUT_KEY_F8: return UKey::UK_F8;
		case GLUT_KEY_F9: return UKey::UK_F9;
		case GLUT_KEY_F10: return UKey::UK_F10;
		case GLUT_KEY_F11: return UKey::UK_F11;
		case GLUT_KEY_F12: return UKey::UK_F12;
		// directional keys
		case GLUT_KEY_LEFT: return UKey::UK_LEFT;
		case GLUT_KEY_UP: return UKey::UK_UP;
		case GLUT_KEY_RIGHT: return UKey::UK_RIGHT;
		case GLUT_KEY_DOWN: return UKey::UK_DOWN;
		case GLUT_KEY_PAGE_UP: return UKey::UK_PAGEUP;
		case GLUT_KEY_PAGE_DOWN: return UKey::UK_PAGEDOWN;
		case GLUT_KEY_HOME: return UKey::UK_HOME;
		case GLUT_KEY_END: return UKey::UK_END;
		case GLUT_KEY_INSERT: return UKey::UK_INSERT;
		default: return UKey::UK_UNDEFINED;
	}
	return UKey::UK_UNDEFINED;
}

UMod_t
mapGlutButton(int button) {
	switch (button) {
		case GLUT_LEFT_BUTTON:
			return UMod::LeftButton;
			break;
		case GLUT_MIDDLE_BUTTON:
			return UMod::MiddleButton;
			break;
		case GLUT_RIGHT_BUTTON:
			return UMod::RightButton;
			break;
	}
	return UMod::NoButton;
}

void
mouseMotionFunc(int x, int y) {
	display->pushMouseMove(context, x, y);
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}

void
mouseButtonFunc(int button, int updown, int x, int y) {
	if (updown == GLUT_DOWN) {
		display->pushMouseButtonDown(context,
			x, y, mapGlutButton(button));
	} else {
		display->pushMouseButtonUp(context,
			x, y, mapGlutButton(button));
	}
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}

void
keyboardFunc(unsigned char key, int, int) {
	display->pushKeyDown(context, UKeyCode_t(key), key);
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}
void
keyboardUpFunc(unsigned char key, int, int) {
	display->pushKeyUp(context, UKeyCode_t(key), key);
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}

void
specialFunc(int key, int, int) {
	display->pushKeyDown(context, mapGlutSpecialKey(key), key);
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}
void
specialUpFunc(int key, int, int) {
	display->pushKeyUp(context, mapGlutSpecialKey(key), key);
	if (context->needsRepaint()) {
		glutPostRedisplay();
	}
}


#else

int main(int argc, char ** argv) {
	std::cerr << "Glut at compile time not found.\n";
	return 0;
}
#endif // HAVE_GLUT_H
