/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/internalframe.cpp
    begin             : Sat Jan 17 2004
    $Id: internalframe.cpp,v 1.5 2005/02/14 10:48:16 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows how to use internal frames

using namespace ufo;

void createContent(UWidget * content);
void createFrame(URootPane * root);
void create2ndFrame(URootPane * root);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("internal frames");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// create some widgets and add them to the content pane
	createContent(frame->getContentPane());
	createFrame(frame->getRootPane());
	create2ndFrame(frame->getRootPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
	content->add(new UButton("useless button"));
}

void
frameEvents(UWidgetEvent * e) {
	switch (e->getType()) {
		case UEvent::WidgetShown:
			std::cerr << "frame shown\n";
			break;
		case UEvent::WidgetHidden:
			std::cerr << "frame hidden\n";
			break;
		case UEvent::WidgetResized:
			std::cerr << "frame resized\n";
			break;
		case UEvent::WidgetMoved:
			std::cerr << "frame moved\n";
			break;
	}
}

void
createFrame(URootPane * root) {
	UInternalFrame * iframe = new UInternalFrame("frame");

	// we need to set the size and position explicitely as
	// internal frames panel does not have layout managers
	iframe->setBounds(100, 100, 220, 220);

	// add the frame to the root pane
	root->addFrame(iframe);

	// add some frame signals
	iframe->sigWidgetShown().connect(slot(&frameEvents));
	iframe->sigWidgetHidden().connect(slot(&frameEvents));
	iframe->sigWidgetResized().connect(slot(&frameEvents));
	iframe->sigWidgetMoved().connect(slot(&frameEvents));

	// some content

	// add a label to the internal frame
	iframe->getContentPane()->setLayout(new UFlowLayout());
	iframe->getContentPane()->add(new ULabel("bla bla"));
	UButton * button = new UButton("button");
	iframe->getContentPane()->add(button);

	// internal frames can have menus
	UMenuBar * mbar = new UMenuBar();
	iframe->getRootPane()->setMenuBar(mbar);

	UMenu * menu = new UMenu("file");
	mbar->add(menu);
	menu->add(new UMenuItem("useless menu item"));
}

void
create2ndFrame(URootPane * root) {
	UInternalFrame * iframe = new UInternalFrame("frame 2");

	// we need to set the size and position explicitely as
	// internal frames panel does not have layout managers
	iframe->setBounds(300, 200, 220, 220);

	// add the frame to the root pane
	root->addFrame(iframe);

	// add a label to the internal frame
	iframe->getContentPane()->setLayout(new UFlowLayout());
	iframe->getContentPane()->add(new ULabel("bla bla"));
	iframe->getContentPane()->add(new UButton("useless button"));

	root->moveToFront(iframe);
}
