/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/menus.cpp
    begin             : Wed Mar 26 2003
    $Id: menus.cpp,v 1.6 2005/06/21 09:28:47 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// Menu example

using namespace ufo;

void createContent(UWidget * content);

// menus are added to the root pane's menu bar
void createMenu(URootPane * rootPane);

int
main(int argc, char ** argv) {
	// Creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("menus and sub-menus");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// create some widgets and add them to the content pane
	createContent(frame->getContentPane());

	// add the menu bar to the root pane
	createMenu(frame->getRootPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
	content->add(new UButton("&button"));
	UButton * button = new UButton("disabled button");
	button->setEnabled(false);
	content->add(button);
}

// this method is used as slot for a menu item
void
print_blah(UActionEvent * e) {
	// If unspecified, the action command equals to the menu text
	// You can set the action command via
	//  {menu,button}->setActionCommand(string)
	// This way, you can easily connect multiple buttons to one
	// action slot.
	std::cout << "blah from " << e->getActionCommand() << "\n";
}

void
exit_ufo(UActionEvent * e) {
	// Send a quit event to the UFO event queue.
	// UDisplay::dispatchEvents() should return true.
	UDisplay::getDefault()->pushEvent(new UQuitEvent());
}

void
createMenu(URootPane * rootPane) {
	// first, we have to create a menu bar
	UMenuBar * mbar = new UMenuBar();

	// add the menu bar to the root pane
	rootPane->setMenuBar(mbar);

	// lets add some menus to the new menu bar
	UMenu * menu = new UMenu("&file");

	// add the menu to the menu bar
	mbar->add(menu);

	// a menu can have several menu items
	menu->add(new UMenuItem("useless menu item"));

	// you can connect the menu item with an slot
	// (see buttons.cpp and signals.cpp)

	UMenuItem * item = new UMenuItem("print &blah");
	item->sigActivated().connect(slot(&print_blah));
	menu->add(item);

	// add a separator to the menu
	menu->addSeparator();

	UMenuItem * exit = new UMenuItem("quit");
	exit->sigActivated().connect(slot(&exit_ufo));
	menu->add(exit);

	//
	// second menu
	UMenu * menu2 = new UMenu("&disabled menu");
	menu2->setEnabled(false);
	mbar->add(menu2);

	item = new UMenuItem("useless menu item2");
	item->setEnabled(false);
	menu2->add(new UMenuItem("useless menu item"));
	menu2->add(item);
	menu2->add(new UMenuItem("useless menu item3"));

	//
	// 3rd menu
	UMenu * menu3 = new UMenu("menu with &translucent popup");
	mbar->add(menu3);
	menu3->add(new UMenuItem("useless menu item"));

	// add a disabled menu item
	item = new UMenuItem("disabled menu item");
	item->setEnabled(false);
	menu3->add(item);

	// This sets the opacity hint used by the Look and feel.
	// Warning: experimental
	menu3->getPopupMenu()->setOpacity(0.7f);

	//
	// sub menus
	//
	// just add a menu to an existing menu and you have sub-menus

	menu3->addSeparator();

	UMenu * submenu = new UMenu("sub-menu");
	// add it to the top-level menu
	menu3->add(submenu);

	UMenuItem * item2 = new UMenuItem("print blah 2");
	item2->sigActivated().connect(slot(&print_blah));
	submenu->add(item2);


	// a sub-sub-menu
	UMenu * subsubmenu = new UMenu("sub-sub-menu");
	submenu->add(subsubmenu);

	UMenuItem * item3 = new UMenuItem("print blah 3");
	item3->sigActivated().connect(slot(&print_blah));
	subsubmenu->add(item3);
}
