/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/multipeframes.cpp
    begin             : Fri Aug 13 2004
    $Id: multipleframes.cpp,v 1.3 2005/02/14 10:44:53 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

using namespace ufo;

// This example shows how to create multiple top-level frames.

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("frame1");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// create a frame and load the video plug-in
	UXFrame * frame2 = display.createFrame();
	if (!frame2) {
		std::cerr << "The current video driver does "
			<< "not support multiple frames.\n";
	} else {
		frame2->setTitle("frame 2");
		frame2->setBounds(500, 0, 640, 480);
		frame2->setVisible(true);
	}

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// create some widgets and add them to the content pane
	createContent(frame->getContentPane());
	if (frame2) {
		createContent(frame2->getContentPane());
	}

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			if (frame2) {
				frame2->repaint();
			}
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
	content->add(new UButton("button"));
}
