/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/sdl.cpp
    begin             : Wed Mar 26 2003
    $Id: sdl.cpp,v 1.6 2005/05/21 15:23:42 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows how to use SDL with
// full control over the SDL event loop

#ifdef HAVE_SDL

// This is for convenience.
// The UXSDLDriver provides a static function for
// SDL to UFO event conversion.
#include <ufo/ux/uxsdldriver.hpp>

#include "SDL.h"
#include "GL/gl.h"

using namespace ufo;

void createContent(ufo::UWidget * content);

int
main(int argc, char ** argv) {
	// Init SDL
	SDL_Init(SDL_INIT_VIDEO);

	// libUFO likes unicode values for key type events
	SDL_EnableUNICODE(1);

	// Enable key repeat
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,
		SDL_DEFAULT_REPEAT_INTERVAL);

	// Set OpenGL attributes
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// create SDL frame
	SDL_SetVideoMode(640, 480, 0, SDL_OPENGL | SDL_RESIZABLE);

	// Init UFO
	UXToolkit tk;
	// Init the display with a dummy driver
	// Creates the UFO event queue and usually polls native events
	UXDisplay display("dummygl");

	// You must have a valid OpenGL context to create a UFO context.
	// The UFO context is responsible for UFO widgets and event processing
	UXContext context(
		// bounds of the OpenGL device
		URectangle(0, 0, 640, 480),
		// desired bounds of the UFO context (may differ from above values)
		URectangle(20, 20, 400, 400)
	);
	// You may also create a second context object
	// Uncomment if you want two UFO contexts
	//UXContext context2(URectangle(0, 0, 640, 480),URectangle(440, 50, 120, 400));


	createContent(context.getRootPane()->getContentPane());
	// Uncomment if you want two UFO contexts
	//createContent(context2.getRootPane()->getContentPane());

#define MAX_EVENTS 16
	static SDL_Event events[MAX_EVENTS];

	int numEvents = 0;

	// SDL event loop
	bool done = false;
	while (!done) {
		SDL_PumpEvents();
		// get SDL events from the event queue
		numEvents = SDL_PeepEvents(events, MAX_EVENTS, SDL_GETEVENT, 0xffffff);

		// this is for SDL event preprocessing
		for (int i = 0; i < numEvents; i++) {
			switch (events[i].type) {
				case SDL_KEYDOWN:
				case SDL_KEYUP:
					if (events[i].key.keysym.sym == SDLK_ESCAPE) {
						// quit on escape
						done = true;
					}
					std::cout << "SDL key event received\n";
				break;
				case SDL_MOUSEBUTTONDOWN:
				case SDL_MOUSEBUTTONUP:
					std::cout << "SDL mouse button event received\n";
				break;
				case SDL_MOUSEMOTION:
					//std::cout << "SDL mouse motion event received\n";
				break;
				case SDL_VIDEORESIZE: {
					// This resets internal SDL states and UFO device bounds
					SDL_Surface * screen = SDL_GetVideoSurface();
					SDL_SetVideoMode(events[i].resize.w, events[i].resize.h,
						screen->format->BitsPerPixel, screen->flags);

					// If your UFO context bounds depend on the frame size,
					// you may change it here
					// context.setContextBounds(URectangle(0, 0,
					//	events[i].resize.w, events[i].resize.h));
					}
				break;
				case SDL_VIDEOEXPOSE:
					// mark the root pane to be repainted
					context.getRootPane()->repaint();
					//context2.getRootPane()->repaint();
				break;
				default:
				break;
			}
		}

		// Push all to the UFO event queue
		// This is a static method in the sdl device
		//  (usually only for internal use)
		UXSDLDriver::pushSDLEvents(
			events,  // array of events
			numEvents // event count
		);

		if (!done) {
			// dispatch events on the event queue
			// dispatchEvents return true when a quit was requested
			done = display.dispatchEvents();

			if (context.needsRepaint()) {
				glClearColor(0.0f, 0.0f, 0.0f, 1.0f) ;
				glClear(GL_COLOR_BUFFER_BIT);

				// do your custom drawing here

				// repaint the context
				context.repaint();
				// Uncomment if you want two UFO contexts
				//context2.repaint();
			}

			// swap opengl buffers
			SDL_GL_SwapBuffers();
			// give other programs CPU time
			SDL_Delay(1);
		}
	}
	// quit SDL
	SDL_Quit();

	return EXIT_SUCCESS;
}

void
createContent(ufo::UWidget * content) {
	// we are using the flow layout as layout manager
	content->setLayout(new ufo::UFlowLayout());

	// add a hello world label (static text)
	content->add(new ULabel("Hello, World!"));
	content->add(new UButton("button"));
}

#else

int main(int argc, char ** argv) {
	std::cerr << "SDL not found (HAVE_SDL not defined).\n";
	return 0;
}
#endif // HAVE_SDL
