/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/text.cpp
    begin             : Wed Mar 26 2003
    $Id: text.cpp,v 1.6 2005/09/15 16:22:04 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

// This example shows the dynamic text widget UTextEdit

using namespace ufo;

void createContent(UWidget * content);

int
main(int argc, char ** argv) {
	// creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// creates a frame
	UXFrame * frame = display.createFrame();
	frame->setTitle("text widgets");
	frame->setBounds(0, 0, 640, 480);
	frame->setVisible(true);

	// Now you have a valid OpenGL context

	// user created widgets are placed in the content pane
	// create some widgets and add them to the content pane
	createContent(frame->getContentPane());

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
print_action_command(UActionEvent * e) {
	std::cout << "got action event with command: "
	<< e->getActionCommand() << "\n";
}

void
createContent(UWidget * content) {
	// we are using the flow layout as layout manager
	//content->setLayout(new UFlowLayout());
	content->setOrientation(Vertical);
	
	UWidget * hbox = new UWidget();
	hbox->setOrientation(Horizontal);
	content->add(hbox);

	//
	// create a static text widget
	//
	UTextWidget * staticText = new UTextWidget();
	hbox->add(staticText);

	// set the text
	staticText->setText("hello, world!\nThis is a static text widget\n");
	// you can also use the stream operator
	*staticText << "<< operator\n"
	<< "blah blah\n";

	//
	// create the text edit field
	//
	UTextEdit * text = new UTextEdit();
	hbox->add(text);

	// set some size hints
	text->setColumns(15);
	text->setRows(15);

	// set the text
	text->setText("hello, world!\nThis is a simple text widget\n");

	//
	// make new line
	//
	hbox = new UWidget();
	hbox->setOrientation(Horizontal);
	content->add(hbox);

	//
	// line edit
	//
	ULineEdit * lineEdit = new ULineEdit();
	lineEdit->setText("25 fd d8245 24 h asg");
	lineEdit->sigActivated().connect(slot(&print_action_command));

	hbox->add(new ULabel("&line edit", lineEdit));
	hbox->add(lineEdit);

	//
	// line edit which accepts int vals
	// validation takes place on pressing enter
	//

	ULineEdit * intField = new ULineEdit();
	hbox->add(new ULabel("&integer edit", intField));
	hbox->add(intField);
	intField->setValidator(new UIntValidator());
	intField->setText("25234134");
	intField->sigActivated().connect(slot(&print_action_command));


	//
	// line edit which accepts double vals
	// validation takes place on pressing enter
	//

	ULineEdit * doubleField = new ULineEdit();
	hbox->add(new ULabel("&double edit", doubleField));
	hbox->add(doubleField);
	doubleField->setValidator(new UDoubleValidator());
	doubleField->setText("25234.234");
	doubleField->sigActivated().connect(slot(&print_action_command));
}

