/***************************************************************************
    LibUFO - UI For OpenGL
    copyright         : (C) 2001-2005 by Johannes Schmidt
    email             : schmidtjf at users.sourceforge.net
                             -------------------

    file              : test/xul.cpp
    begin             : Wed Mar 26 2003
    $Id: xul.cpp,v 1.3 2005/09/02 13:29:06 schmidtjf Exp $
 ***************************************************************************/


#include <ufo/ufo.hpp>
#include <ufo/ux/ux.hpp>

using namespace ufo;

// This example shows the use of the XUL parser

void
connect_signals(UXul & uxul);

int
main(int argc, char ** argv) {
	// Creates the toolkit object, initializes UFO
	UXToolkit tk;
	// loads the video driver and creates the event queue
	UXDisplay display;

	// Reads a XUL file
	UXul uxul("gui.xul");
	// Creates a XUL frame
	UXFrame * frame = uxul.createFrame();
	if (!frame) {
		std::cerr << "Please ensure that you have gui.xul "
		<< "in the current directory\n";
		return -1;
	}
	frame->setVisible(true);
	// you can also create only the root pane from a xul file:
	// URootPane * root = uxul.createRootPane();
	// UXFrame * frame = display.createFrame();
	// frame->setBounds(0, 0, 640, 480);
	// frame->setVisible(true);
	// frame->getContext()->setRootPane(root);

	connect_signals(uxul);

	// main event loop
	bool done = false;
	while (!done) {
		// pumps events from the system event queue
		display.pumpEvents();
		// dispatchEvents returns true when a quit event was processed.
		done = display.dispatchEvents();

		if (!done) {
			// repaint the frame
			frame->repaint();
			// give other processes a chance
			tk.sleep(1);
		}
	}

	return 0;
}

void
print_blah(UActionEvent * e) {
	std::cerr << "print blah from " << e->getActionCommand() << "\n";
}

void
my_quit(UActionEvent * e) {
	UDisplay::getDefault()->pushEvent(new UQuitEvent());
}

void
change_color(UActionEvent * e) {
	URadioButton * b = dynamic_cast<URadioButton*>(e->getSource());
	if (b->isSelected()) {
		if (e->getActionCommand() == "Red") {
			b->getParent()->setBackgroundColor(UColor::red);
		} else if (e->getActionCommand() == "Green") {
			b->getParent()->setBackgroundColor(UColor::green);
		}
	}
}

static UProgressBar * progress = NULL;
void
slider_changed(UAbstractSlider * slider) {
	if (progress) {
		progress->setValue(slider->getValue());
	}
}

void
connect_signals(UXul & uxul) {
	// Connects some signals to created widgets
	// This is basically the same as in other examples.
	// Just retrieve your widget via uxul.get and connect the signals

	// connect a button with a signal
	UButton * button = dynamic_cast<UButton*>(uxul.get("print_blah"));
	if (button) {
		button->sigActivated().connect(slot(print_blah));
	}
	// connect a menu item with a signal
	UMenuItem * item = dynamic_cast<UMenuItem*>(uxul.get("file-menu-quit"));
	if (item) {
		item->sigActivated().connect(slot(my_quit));
	}

	// connect radio buttons with signals
	URadioButton * radio = dynamic_cast<URadioButton*>(uxul.get("radio-green"));
	if (radio) {
		radio->sigActivated().connect(slot(change_color));
	}
	radio = dynamic_cast<URadioButton*>(uxul.get("radio-red"));
	if (radio) {
		radio->sigActivated().connect(slot(change_color));
	}

	// connect slider to progress bar
	USlider * slider = dynamic_cast<USlider*>(uxul.get("slider"));
	progress = dynamic_cast<UProgressBar*>(uxul.get("progress"));
	if (slider) {
		slider->sigValueChanged().connect(slot(slider_changed));
	}
}
